{-
   Jitter.hs  (adapted from jitter.h which is (c) Silicon Graphics, Inc)
   This file is part of HOpenGL - a binding of OpenGL and GLUT for Haskell.
   Copyright (C) 2000  Sven Panne <Sven.Panne@informatik.uni-muenchen.de>

   This file contains jitter point arrays for 2,3,4,8,15,24 and 66 jitters.

   The lists are named j2, j3, etc. Each element in the list has the form,
   for example, JitterPoint x y.

   Values are floating point in the range -.5 < x < .5, -.5 < y < .5, and
   have a gaussian distribution around the origin.

   Use these to do model jittering for scene anti-aliasing and view volume
   jittering for depth of field effects. Use in conjunction with the 
   accwindow routine.
-}

module Jitter where

import GL ( GLfloat )

data JitterPoint = JitterPoint GLfloat GLfloat

j2, j3, j4, j8, j15, j24, j66 :: [JitterPoint]

-- 2 jitter points
j2  = [ JitterPoint   0.246490    0.249999  ,
        JitterPoint (-0.246490) (-0.249999) ]

-- 3 jitter points
j3  = [ JitterPoint (-0.373411) (-0.250550) ,
        JitterPoint   0.256263    0.368119  ,
        JitterPoint   0.117148  (-0.117570) ]

-- 4 jitter points
j4  = [ JitterPoint (-0.208147)   0.353730  ,
        JitterPoint   0.203849  (-0.353780) ,
        JitterPoint (-0.292626) (-0.149945) ,
        JitterPoint   0.296924    0.149994  ]

-- 8 jitter points
j8  = [ JitterPoint (-0.334818)   0.435331  ,
        JitterPoint   0.286438  (-0.393495) ,
        JitterPoint   0.459462    0.141540  ,
        JitterPoint (-0.414498) (-0.192829) ,
        JitterPoint (-0.183790)   0.082102  ,
        JitterPoint (-0.079263) (-0.317383) ,
        JitterPoint   0.102254    0.299133  ,
        JitterPoint   0.164216  (-0.054399) ]

-- 15 jitter points
j15 = [ JitterPoint   0.285561    0.188437  ,
        JitterPoint   0.360176  (-0.065688) ,
        JitterPoint (-0.111751)   0.275019  ,
        JitterPoint (-0.055918) (-0.215197) ,
        JitterPoint (-0.080231) (-0.470965) ,
        JitterPoint   0.138721    0.409168  ,
        JitterPoint   0.384120    0.458500  ,
        JitterPoint (-0.454968)   0.134088  ,
        JitterPoint   0.179271  (-0.331196) ,
        JitterPoint (-0.307049) (-0.364927) ,
        JitterPoint   0.105354  (-0.010099) ,
        JitterPoint (-0.154180)   0.021794  ,
        JitterPoint (-0.370135) (-0.116425) ,
        JitterPoint   0.451636  (-0.300013) ,
        JitterPoint (-0.370610)   0.387504  ]

-- 24 jitter points
j24 = [ JitterPoint   0.030245    0.136384  ,
        JitterPoint   0.018865  (-0.348867) ,
        JitterPoint (-0.350114) (-0.472309) ,
        JitterPoint   0.222181    0.149524  ,
        JitterPoint (-0.393670) (-0.266873) ,
        JitterPoint   0.404568    0.230436  ,
        JitterPoint   0.098381    0.465337  ,
        JitterPoint   0.462671    0.442116  ,
        JitterPoint   0.400373  (-0.212720) ,
        JitterPoint (-0.409988)   0.263345  ,
        JitterPoint (-0.115878) (-0.001981) ,
        JitterPoint   0.348425  (-0.009237) ,
        JitterPoint (-0.464016)   0.066467  ,
        JitterPoint (-0.138674) (-0.468006) ,
        JitterPoint   0.144932  (-0.022780) ,
        JitterPoint (-0.250195)   0.150161  ,
        JitterPoint (-0.181400) (-0.264219) ,
        JitterPoint   0.196097  (-0.234139) ,
        JitterPoint (-0.311082) (-0.078815) ,
        JitterPoint   0.268379    0.366778  ,
        JitterPoint (-0.040601)   0.327109  ,
        JitterPoint (-0.234392)   0.354659  ,
        JitterPoint (-0.003102) (-0.154402) ,
        JitterPoint   0.297997  (-0.417965) ]

-- 66 jitter points
j66 = [ JitterPoint   0.266377  (-0.218171) ,
        JitterPoint (-0.170919) (-0.429368) ,
        JitterPoint   0.047356  (-0.387135) ,
        JitterPoint (-0.430063)   0.363413  ,
        JitterPoint (-0.221638) (-0.313768) ,
        JitterPoint   0.124758  (-0.197109) ,
        JitterPoint (-0.400021)   0.482195  ,
        JitterPoint   0.247882    0.152010  ,
        JitterPoint (-0.286709) (-0.470214) ,
        JitterPoint (-0.426790)   0.004977  ,
        JitterPoint (-0.361249) (-0.104549) ,
        JitterPoint (-0.040643)   0.123453  ,
        JitterPoint (-0.189296)   0.438963  ,
        JitterPoint (-0.453521) (-0.299889) ,
        JitterPoint   0.408216  (-0.457699) ,
        JitterPoint   0.328973  (-0.101914) ,
        JitterPoint (-0.055540) (-0.477952) ,
        JitterPoint   0.194421    0.453510  ,
        JitterPoint   0.404051    0.224974  ,
        JitterPoint   0.310136    0.419700  ,
        JitterPoint (-0.021743)   0.403898  ,
        JitterPoint (-0.466210)   0.248839  ,
        JitterPoint   0.341369    0.081490  ,
        JitterPoint   0.124156  (-0.016859) ,
        JitterPoint (-0.461321) (-0.176661) ,
        JitterPoint   0.013210    0.234401  ,
        JitterPoint   0.174258  (-0.311854) ,
        JitterPoint   0.294061    0.263364  ,
        JitterPoint (-0.114836)   0.328189  ,
        JitterPoint   0.041206  (-0.106205) ,
        JitterPoint   0.079227    0.345021  ,
        JitterPoint (-0.109319) (-0.242380) ,
        JitterPoint   0.425005  (-0.332397) ,
        JitterPoint   0.009146    0.015098  ,
        JitterPoint (-0.339084) (-0.355707) ,
        JitterPoint (-0.224596) (-0.189548) ,
        JitterPoint   0.083475    0.117028  ,
        JitterPoint   0.295962  (-0.334699) ,
        JitterPoint   0.452998    0.025397  ,
        JitterPoint   0.206511  (-0.104668) ,
        JitterPoint   0.447544  (-0.096004) ,
        JitterPoint (-0.108006) (-0.002471) ,
        JitterPoint (-0.380810)   0.130036  ,
        JitterPoint (-0.242440)   0.186934  ,
        JitterPoint (-0.200363)   0.070863  ,
        JitterPoint (-0.344844) (-0.230814) ,
        JitterPoint   0.408660    0.345826  ,
        JitterPoint (-0.233016)   0.305203  ,
        JitterPoint   0.158475  (-0.430762) ,
        JitterPoint   0.486972    0.139163  ,
        JitterPoint (-0.301610)   0.009319  ,
        JitterPoint   0.282245  (-0.458671) ,
        JitterPoint   0.482046    0.443890  ,
        JitterPoint (-0.121527)   0.210223  ,
        JitterPoint (-0.477606) (-0.424878) ,
        JitterPoint (-0.083941) (-0.121440) ,
        JitterPoint (-0.345773)   0.253779  ,
        JitterPoint   0.234646    0.034549  ,
        JitterPoint   0.394102  (-0.210901) ,
        JitterPoint (-0.312571)   0.397656  ,
        JitterPoint   0.200906    0.333293  ,
        JitterPoint   0.018703  (-0.261792) ,
        JitterPoint (-0.209349) (-0.065383) ,
        JitterPoint   0.076248    0.478538  ,
        JitterPoint (-0.073036) (-0.355064) ,
        JitterPoint   0.145087    0.221726  ]
